@@ -72,12 +72,10 @@ pub struct EntityAggregator {
 }
 
 impl EntityAggregator {
-    pub fn new<I>(mutations_metadata: I) -> Result<EntityAggregator, Error>
+    pub fn new<I>(sorted_mutations: I) -> EntityAggregator
     where
         I: Iterator<Item = MutationMetadata>,
     {
-        let ordered_mutations_metadata = sort_mutations_commit_time(mutations_metadata);
-
         let mut entity_id = String::new();
         let mut entity_creation_date = None;
         let mut entity_modification_date = None;
@@ -89,11 +87,20 @@ impl EntityAggregator {
         let mut active_operation_ids = HashSet::<OperationId>::new();
         let mut last_operation_id = None;
         let mut last_block_offset = None;
+        let mut prev_dedup_operation_id = None;
         let mut in_pending = false;
         let mut pending_deletion = false;
         let mut mutation_count = 0;
 
-        for current_mutation in ordered_mutations_metadata {
+        for current_mutation in sorted_mutations {
+            assert_ne!(
+                Some(current_mutation.operation_id),
+                prev_dedup_operation_id,
+                "got two mutation with same operation id: mutation={:?}",
+                current_mutation,
+            );
+            prev_dedup_operation_id = Some(current_mutation.operation_id);
+
             mutation_count += 1;
 
             let current_operation_id = current_mutation.operation_id;
@@ -199,7 +206,7 @@ impl EntityAggregator {
             .values()
             .any(|t| t.deletion_date.is_none() && t.has_reference);
 
-        Ok(EntityAggregator {
+        EntityAggregator {
             entity_id,
             traits,
             active_operations: active_operation_ids,
@@ -213,7 +220,7 @@ impl EntityAggregator {
             in_pending,
             pending_deletion,
             has_reference,
-        })
+        }
     }
 
     /// Annotates each trait with projections that are matching them in a query.
@@ -401,17 +408,6 @@ pub fn project_trait_fields(
     Ok(())
 }
 
-/// Sorts mutations in order they got committed (block offset/pending, then
-/// operation id)
-pub fn sort_mutations_commit_time<M: Iterator<Item = MutationMetadata>>(
-    mutations: M,
-) -> impl Iterator<Item = MutationMetadata> {
-    mutations.sorted_by_key(|result| {
-        let block_offset = result.block_offset.unwrap_or(std::u64::MAX);
-        (block_offset, result.operation_id)
-    })
-}
-
 fn update_if_newer(current: &mut Option<DateTime<Utc>>, new: Option<DateTime<Utc>>) {
     if current.is_none() || new > *current {
         *current = new;
@@ -456,7 +452,7 @@ pub(crate) mod tests {
             mock_put_trait(&t2, TYPE1, Some(2), 4, None, None),
         ];
 
-        let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+        let em = EntityAggregator::new(mutations.into_iter());
         assert_eq!(em.traits.get(&t1).unwrap().last_operation_id, Some(3));
         assert_eq!(em.traits.get(&t2).unwrap().last_operation_id, Some(5));
         assert_eq!(em.traits.get(&t3).unwrap().last_operation_id, Some(6));
@@ -477,7 +473,7 @@ pub(crate) mod tests {
         ];
 
         // operation 1 should be discarded, and only operation 2 active
-        let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+        let em = EntityAggregator::new(mutations.into_iter());
         assert_eq!(em.traits.get(&t1).unwrap().last_operation_id, Some(2));
         assert!(em.active_operations.contains(&2));
     }
@@ -491,7 +487,7 @@ pub(crate) mod tests {
             mock_delete_trait(&t1, Some(2), 2),
         ];
 
-        let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+        let em = EntityAggregator::new(mutations.into_iter());
         assert!(em.deletion_date.is_some());
         assert!(em.traits.get(&t1).unwrap().deletion_date.is_some());
         assert!(em.active_operations.contains(&2));
@@ -509,7 +505,7 @@ pub(crate) mod tests {
 
         // delete operation should be discarded since an operation happened on the trait
         // before it got committed
-        let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+        let em = EntityAggregator::new(mutations.into_iter());
         assert_eq!(em.traits.get(&t1).unwrap().last_operation_id, Some(2));
         assert!(em.active_operations.contains(&2))
     }
@@ -523,7 +519,7 @@ pub(crate) mod tests {
             mock_delete_entity(Some(2), 2),
         ];
 
-        let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+        let em = EntityAggregator::new(mutations.into_iter());
         assert!(em.deletion_date.is_some());
         assert!(em.traits.get(&t1).unwrap().deletion_date.is_some());
         assert!(em.active_operations.contains(&2));
@@ -539,7 +535,7 @@ pub(crate) mod tests {
             mock_put_trait(&t1, TYPE1, Some(2), 1, None, None),
         ];
 
-        let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+        let em = EntityAggregator::new(mutations.into_iter());
         assert_eq!(em.traits.get(&t1).unwrap().last_operation_id, Some(1));
         assert!(em.active_operations.contains(&1));
     }
@@ -549,7 +545,7 @@ pub(crate) mod tests {
         let t1 = "t1".to_string();
 
         let merge_mutations = |mutations: Vec<MutationMetadata>| -> TraitAggregator {
-            let mut em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let mut em = EntityAggregator::new(mutations.into_iter());
             em.traits.remove(&t1).unwrap()
         };
 
@@ -654,14 +650,14 @@ pub(crate) mod tests {
         {
             // creation date based on operation
             let mutations = vec![mock_put_trait(&t1, TYPE1, Some(2), 1, None, None)];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let em = EntityAggregator::new(mutations.into_iter());
             assert_dates(&em, Some(1), None, None);
         }
 
         {
             // explicit dates
             let mutations = vec![mock_put_trait(&t1, TYPE1, Some(2), 10, Some(1), Some(2))];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let em = EntityAggregator::new(mutations.into_iter());
             assert_dates(&em, Some(1), Some(2), None);
         }
 
@@ -671,7 +667,7 @@ pub(crate) mod tests {
                 mock_put_trait(&t1, TYPE1, Some(2), 10, None, None),
                 mock_put_trait(&t1, TYPE1, Some(3), 11, None, None),
             ];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let em = EntityAggregator::new(mutations.into_iter());
             assert_dates(&em, Some(10), Some(11), None);
         }
 
@@ -679,11 +675,11 @@ pub(crate) mod tests {
             // trait deletion counts as modification
             let mutations = vec![
                 mock_put_trait(&t1, TYPE1, Some(2), 10, None, None),
-                mock_put_trait(&t2, TYPE1, Some(2), 10, None, None),
-                mock_delete_trait(&t1, Some(3), 11),
+                mock_put_trait(&t2, TYPE1, Some(2), 11, None, None),
+                mock_delete_trait(&t1, Some(3), 12),
             ];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
-            assert_dates(&em, Some(10), Some(11), None);
+            let em = EntityAggregator::new(mutations.into_iter());
+            assert_dates(&em, Some(10), Some(12), None);
         }
 
         {
@@ -692,7 +688,7 @@ pub(crate) mod tests {
                 mock_put_trait(&t1, TYPE1, Some(2), 10, None, None),
                 mock_delete_trait(&t1, Some(3), 11),
             ];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let em = EntityAggregator::new(mutations.into_iter());
             assert_dates(&em, None, None, Some(11));
         }
         {
@@ -701,7 +697,7 @@ pub(crate) mod tests {
                 mock_put_trait(&t1, TYPE1, Some(2), 10, None, None),
                 mock_delete_entity(Some(2), 11),
             ];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let em = EntityAggregator::new(mutations.into_iter());
             assert_dates(&em, None, None, Some(11));
         }
 
@@ -712,7 +708,7 @@ pub(crate) mod tests {
                 mock_delete_entity(Some(2), 2),
                 mock_put_trait(&t1, TYPE1, Some(3), 3, Some(20), Some(21)),
             ];
-            let em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let em = EntityAggregator::new(mutations.into_iter());
             assert_dates(&em, Some(20), Some(21), None);
         }
     }
@@ -780,7 +776,7 @@ pub(crate) mod tests {
                 mock_put_trait(t1, TYPE1, Some(1), 1, None, None),
                 mock_put_trait(t2, TYPE2, Some(1), 2, None, None),
             ];
-            let mut em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let mut em = EntityAggregator::new(mutations.into_iter());
             em.annotate_projections(&[Projection {
                 package: vec!["exocore.test".to_string()],
                 field_ids: vec![1],
@@ -797,7 +793,7 @@ pub(crate) mod tests {
                 mock_put_trait(t1, TYPE1, Some(1), 1, None, None),
                 mock_put_trait(t2, TYPE2, Some(1), 2, None, None),
             ];
-            let mut em = EntityAggregator::new(mutations.into_iter()).unwrap();
+            let mut em = EntityAggregator::new(mutations.into_iter());
             em.annotate_projections(&[
                 Projection {
                     package: vec![format!("{}$", TYPE1)],
